#!/usr/bin/python

#This program reads a data-file, and plots it. c.f. fftplot.py
#If the file has several columns, we can choose any one or the sum, or all of them.

#Copyright (C) Richard Neill 2011-2012, <maths at REMOVE.ME.richardneill.org>. This program is Free Software. You can
#redistribute and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation,
#either version 3 of the License, or (at your option) any later version. There is NO WARRANTY, neither express nor implied.
#For the details, please see: http://www.gnu.org/licenses/gpl.html


#Begin
from __future__ import division		#Needed, as this may run under Python 2.x
import getopt, sys, os, string
import numpy as numpy
from pylab import *
from textwrap import *

#Help
def usage():
	print 	"dataplot:  plot one or all columns of a data-file. Show summary statistics.\n"
	print 	"USAGE:    " + str(os.path.basename(sys.argv[0])) + "  [OPTS]  infile.dat\n"
	print	"OPTIONS:  -c, --column      N,sum,all   use Nth column's data (N is 0-based). [default: all]."
	print   "          -t, --title       TITLE       override plot title"
	print   "          -x, --xlabel      LABEL       override x-axis label"
	print   "          -y, --xlabel      LABEL       override y-axis label"
	print   "          -u, --ulabel      L,A,BE,LS   override legend labels.\n"
	print   "          -o, --outplot     FILE.png    save plot as FILE.png/.jpg, \"auto\" names it after infile.\n"
	print   "          -n, --noplot                  don't draw the plot window on screen (useful for testing/scripting)."
	print   "          -h, --help                    show help.\n"
	print   "INPUT:    Input file, infile.dat ('-' for stdin) should be columns of (int/float) data, separated by"
	print   "          whitespace, with optional #comments, i.e. the standard form for numpy.loadfile(). cf. genffttest."
	print   "          (compressed files (.gz/.bz2) can also be read transparently, no need to uncompress first.)\n"
	print   "NOTE:     " + str(os.path.basename(sys.argv[0])) + " can crash on larger datasets, especially on older PCs with less RAM. "
	print   "          So, consider decimating the data in advance, or first copying the file to a more powerful machine."
	print   ""
	return

#Initialise.
plotdims=(12, 9); plotdpi=80; 	#Sets initial size of plot on screen, and the dimensions of the saved image.  DPI affects font-size.
title = xlabel = ylabel = ulabel = outplot = noplot = False
column = -1

#Parse args
try:
	opts, args = getopt.gnu_getopt(sys.argv[1:], "c:hnt:x:y:u:o:", [ "column=", "help", "noplot", "title=", "xlabel=", "ylabel=", "ulabel=", "outplot="])
except getopt.GetoptError, err:
	print str(err) + ": use -h for help." ; sys.exit(2)

for o, a in opts:
	if o in ("-h", "--help"):
		usage(); exit(0);
	elif o in ("-c", "--column"):
		if (a == "all") or (a == "sum"):
			column = a
		else:
			column = int(a)
	elif o in ("-n", "--noplot"):
		noplot = True			
	elif o in ("-t", "--title"):
		title = fill(a, 80)       #wrap long lines if necessary (otherwise they get truncated)
	elif o in ("-x", "--xlabel"):
		xlabel = a
	elif o in ("-y", "--ylabel"):
		ylabel = a
	elif o in ("-u", "--ulabel"):
		ulabel = a
	elif o in ("-o", "--outplot"):
		outplot = a
	else:
		assert False, "unhandled option"

if ((len(args) == 0) and ('-' in sys.argv)):		#Ugly hack for older versions.
		infile = sys.stdin
		infilename = "<stdin>"
		if (outplot == "auto"):
			outplot = "stdin.png"
elif (len(args) != 1):
		print "Error: One (non-option) argument is required: input-filename."
		exit (2)
else:	
	infile = args[0]
	infilename = infile

	if (infile == "-"):
		infile = sys.stdin
		infilename = "<stdin>"
		if (outplot == "auto"):
			outplot = "stdin.png"
	else:
		if (outplot == "auto"):
			outplot = os.path.splitext(os.path.basename(infilename))[0] + ".png"


#Load data from file.
try:
	data_in = numpy.loadtxt (infile);				#numpy.loadtxt really is all we need! It even handles comments :-)
except IOError, err:							#the function is very flexible, though the defaults suffice here.
	print "Error reading datafile " + infilename + ": " + str(err)
	exit (1);
except ValueError, err:
	print "Error processing malformed datafile " + infilename + ": " + str(err)
	exit (1);


#Get size of data file, sanity check it.
if (len(data_in.shape) == 1):		#if it is a single column (1d-array),  convert to a 2d-array with 1 column.
	data_in = data_in[:,newaxis]
num_cols = len(data_in[0, ]);
num_rows = len(data_in);
if (num_cols == 0) or (num_cols > 20) or (num_rows < 4) or (num_rows > 1E7):
	print "Data size is out of range. File contains "  + str(num_rows) + " rows of " + str(num_cols) + " columns."
	exit (1)


#Extract the relevant column.
if (column == -1):			#Default: All columns
	column = "all"
	if (num_cols == 1):		#..but if the file only has 1 column, don't make it an array (for clarity).
		column = 0
if (column == "sum"):			#Sum the columns
	data = data_in [ : , 0]
	for col in range (1, num_cols):
		data += data_in [ : , col]
	legend = ["sum (0:"+str(num_cols -1)+")"]
elif (column == "all"):			#All columns (in parallel).
	data = data_in
	legend = ["col 0"]
	for col in range (1, num_cols):
		legend.append ("col " +str(col))
elif (column < num_cols):
	data = data_in [ : , column]	#This pulls out a 1-d array, from a 2-d array.
	legend = ["col "+str(column)]
else:
	assert False, "invalid column specified: " + str(column)


#Sampling number and rate.
num_samples = len(data)			#This many samples.
times =	arange (0, num_samples)		#Create time-series for raw-data x-axis

#Statistics.
y = data
S_y     = sum(y, axis=0)		#Get a 1 or n-element result.
S_yy    = sum((y * y), axis=0)
mean    = S_y / num_samples
stddev  = sqrt (abs( (S_yy / num_samples)  - (mean * mean) ))

#Plot the raw input data.
plt.figure(figsize=plotdims, dpi=plotdpi);
plt.title ("Raw data  [ " + infilename + ", column " + str(column) + " ]")
plt.xlabel("Sample number")
plt.ylabel("Signal")
lines = plt.plot (times, data)	#If data is a multi-column array, this plots multiple lines :-)
plt.setp (lines, marker = '.', linestyle='-');	#Plot with solid lines and small dots.
plt.grid()
if (num_cols > 1):
	plt.legend(legend)

#Override labels, if specified.
if (title != False):
	plt.title (title)
if (xlabel != False):
	plt.xlabel (xlabel)
if (ylabel != False):
	plt.ylabel (ylabel)
if (ulabel != False):
	ulabels = string.split (ulabel, ",", (num_cols - 1) )	#comma separated.
	plt.legend (ulabels)
	if ((column < num_cols) and (len(ulabels) >= column)):  #select nth label if possible.
		plt.legend ([ulabels[column]])


#Show summary info.
print "Plotted column " + str(column) + " from file '" + infilename + "': " + str(num_samples) + " points."
print "n:       ", num_samples
print "mean:    ", mean
print "stddev:  ", stddev
print ""


#Now save and show the plot, and exit.
if (outplot):
	plt.savefig (outplot)
	print "Saved file as: " + outplot
if (noplot == False):
	plt.show()
exit (0)
