/* Generate some test data for FFT analysis. Outputs 4 columns of data (ascii-floating point, tab-separated, newlines, leading header with #) */

/* Copyright (C) Richard Neill 2011-2012, <maths at REMOVE.ME.richardneill.org>. This program is Free Software. You can
 * redistribute and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation,
 * either version 3 of the License, or (at your option) any later version. There is NO WARRANTY, neither express nor implied.
 * For the details, please see: http://www.gnu.org/licenses/gpl.html */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <math.h>

#define  CHANNELS	4		/* Number of channels: 4 columns of data */
#define  SAMPLES	10000		/* Number of samples to create  (may be odd or even) */
#define  PERIOD_US	5		/* Sample interval, in us. Scales the frequency. (cf 200kHz for NI4462) */

#define  PI		M_PI		/* 3.14... */
#define  Hz		* 1
#define  kHz		* 1e3

#define outprintf(...)		fprintf(outfile, __VA_ARGS__)

#define output(...)		fprintf (outfile, "%f\t%f\t%f\t%f\n", __VA_ARGS__);


FILE *outfile;

void printhelp(){
	fprintf (stderr, "This generates some sample data, suitable for testing the FFT plots.\n"
			 "USAGE:   genffttest   --mode  outfile.dat\n"
			 "MODES:   --sincos:      basic waves: sin, cos, sin+cos, cos(2f)\n"
			 "         --cosines:     basic cosines: cos (f), cos(2f), cos(3f), cos(5f)\n"
			 "         --alias:       awkward frequencies: LF, unaligned with bin-freq, > nyquist, > sampling\n"
			 "         --spectrum:    3.cos(2f) + dc, square-wave, square-wave-partialsum, delta-function\n"
			 "         --slopes:      linear, exponential, sawtooth and constant.\n"
			 "         --noise:       cos waves, with various amounts of noise\n"
			"          --dchum        dc, with noise and hum\n"
			 "OUTPUT:  a simple format: 4 colums of ascii, tab-delimited floats, with #comment-lines.\n"
			 "         This format is, conveniently, parsed by Python's numpy.loadtxt(). \n"
			 "\n");
}

void add_titles (char *comment, char *eqn0, char *eqn1, char *eqn2, char *eqn3){
 		fprintf (stderr, "Generating data: %s ...\n", comment);
 		outprintf ("# y0:\t\t%s\n", eqn0);
 		outprintf ("# y1:\t\t%s\n", eqn1);
 		outprintf ("# y2:\t\t%s\n", eqn2);
 		outprintf ("# y3:\t\t%s\n", eqn3);
}

/* Gaussian noise, mean 0, variance 1. */
double noise (){   /* See hawaiisim.c for details */
	int i;
	double z;
	z = 0;
	for (i=0; i<12; i++){
		z += ((double)rand() / RAND_MAX);   /* uniform [0,1] */
	}
	return (z - 6);
}

int main ( int argc, char **argv ) {
	char *outfile_filename, *type, *comment, *eqn0, *eqn1, *eqn2, *eqn3;
	double t, tpt, wt, y0,y1,y2,y3, T;	/* t = time, y_n = output, tpt = (2.PI.t), wt is omega-t == 2.pi.f.t, T = total run time. */
	int i, x;
	srand (time (NULL));			/* Seed random number generator (omit this to make the "random" data reproducable). */

	/* Process options */
	if (argc < 2){
		printhelp();
		exit (1);
	}
	if ((!strcmp (argv[1], "-h")) || (!strcmp (argv[1], "--help"))){
		printhelp();
		exit (1);
	}
	if ( argc != 3 ) {
		fprintf ( stderr, "Syntax: %s --type outfile.dat    #-h for help.\n", argv[0] );
		exit (1);
	}
	type = argv[1];
	outfile_filename = argv[2];

	if ( ( outfile = fopen ( outfile_filename, "w" ) ) == NULL ) {
		fprintf ( stderr, "Could not open %s for writing: %s\n", outfile_filename, strerror ( errno ) );
		exit ( 1 );
	}

	T =  (double)(PERIOD_US * SAMPLES)/1e6;  /* total run time for all the samples */

	/* Output header */
	outprintf ("# This is some sample data for FFT tests:\n");
	outprintf ("# channels:\t%d\n", 		CHANNELS);
	outprintf ("# samples:\t%d\n", 			SAMPLES);
	outprintf ("# sample_interval_us:\t%d\n",	PERIOD_US);
	outprintf ("# sampling_freq_hz:\t%f\n", 	((double)1e6/PERIOD_US));
	outprintf ("# length_s:\t%f\n", T);


	/* Generate some data, and output it. */

	/* Basic Sin and Cos waves */
	if (!strcmp (type, "--sincos")){

		comment = "basic sin, cos, sin+cos, cos(2f)";
		eqn0   =  "y0 = sin(w t),                f = 1 kHz";
		eqn1   =  "y1 = cos(w t),                f = 1 kHz";
		eqn2   =  "y2 = sin(w t) + cos (w t),    f = 1 kHz";
		eqn3   =  "y3 = cos(2w t),               f = 2 kHz";

		add_titles (comment, eqn0, eqn1, eqn2, eqn3);

		for (i = 0 ; i < SAMPLES; i++){
			tpt = (double)(2 * PI * i * PERIOD_US) / 1e6;
			wt  = 1 kHz * tpt;

 			y0 = sin (wt);
 			y1 = cos (wt);
 			y2 = sin (wt) + cos (wt);
 			y3 = cos (2 * wt);

			output (y0, y1, y2, y3);
		}

	/* Cos waves, different frequencies */
	}else if (!strcmp (type, "--cosines")){

		comment = "basic cosines: cos (f), cos(2f), cos(3f), cos(5f)";
		eqn0   =  "y0 = cos(w t),                f  = 1 kHz";
		eqn1   =  "y1 = cos(2 w t),              2f = 2 kHz";
		eqn2   =  "y2 = cos(3 w t),              3f = 3 kHz";
		eqn3   =  "y3 = cos(5 w t),              5f = 5 kHz";

		add_titles (comment, eqn0, eqn1, eqn2, eqn3);

		for (i = 0 ; i < SAMPLES; i++){
			tpt = (double)(2 * PI * i * PERIOD_US) / 1e6;
			wt  = 1 kHz * tpt;

			y0 = cos (wt);
			y1 = cos (2*wt);
			y2 = cos (3*wt);
			y3 = cos (5*wt);

			output (y0, y1, y2, y3);
		}


	/* Test of aliasing, at nasty frequencies. */
	}else if (!strcmp (type, "--alias")){

		comment = "awkward frequencies (incomplete cycles, non-integer-multiple-of f_s, above nyquist, above sampling";
		eqn0   =  "y0 = cos(w t),              f  = 1   Hz";
		eqn1   =  "y1 = cos(w t),              f  = 1.006 kHz  (unaligned with f_s)";
		eqn2   =  "y2 = cos(w t),              f  = 120 kHz";
		eqn3   =  "y3 = cos(w t),              f  = 310 kHz";

		add_titles (comment, eqn0, eqn1, eqn2, eqn3);

		for (i = 0 ; i < SAMPLES; i++){
			tpt = (double)(2 * PI * i * PERIOD_US) / 1e6;
			wt  = 1 kHz * tpt;

			y0 = cos (wt / 1000);
			y1 = cos (1.006 *wt);		/* 1.000, 1.020 are strong, 1.006 is weak */
			y2 = cos (120*wt);
			y3 = cos (310*wt);

			output (y0, y1, y2, y3);
		}


	/* Some things with interesting spectra. */
	}else if (!strcmp (type, "--spectrum")){

		comment = "interesting spectra:  3.cos(2f) +4, square-wave, square-wave-partialsum, pulse";
		eqn0   =  "y0 = 3.cos(2w t) + 4,         f  = 2  kHz";
		eqn1   =  "y1 = square_wave(t),          f  = 1  kHz";
		eqn2   =  "y2 = square-wave-partial-sum, f  = 1  kHz";
		eqn3   =  "y3 = pulse(),                 1 sample, 1000 high";

		add_titles (comment, eqn0, eqn1, eqn2, eqn3);

		for (i = 0 ; i < SAMPLES; i++){
			tpt = (double)(2 * PI * i * PERIOD_US) / 1e6;
			wt  = 1 kHz * tpt;

			y0 = 3 * cos (2*wt) + 4;	/* DC offset should be interesting */

			x = 1e6 / (PERIOD_US * 1 kHz);
			if ( (i % x) < (x / 2) ){	/* 1kHz square wave */
				y1 = 1;
			}else{
				y1 = -1;
			}

			y2 = cos (wt)  - cos(3*wt)/3 + cos(5*wt)/5 - cos(7*wt)/7 + cos(9*wt)/9 - cos(11*wt)/11; /* partial sum of sines for square-wave */

			y3 = 0;
			if (i == SAMPLES/2){		/* single spike. */
				y3 = 1000;
			}

			output (y0, y1, y2, y3);
		}


	/* Linear and Exponential slopes */
	}else if (!strcmp (type, "--slopes")){

		comment = "linear, exponential, sawtooth and constant";
		eqn0   =  "y0 = 1 - 2(t/T)";
		eqn1   =  "y1 = exp (-t/T)";
		eqn2   =  "y2 = 100 Hz sawtooth";
		eqn3   =  "y3 = 1";

		add_titles (comment, eqn0, eqn1, eqn2, eqn3);

		for (i = 0 ; i < SAMPLES; i++){
			t = (double)(i * PERIOD_US) / 1e6;

			y0 = 1 - 2 * (t/T);
			y1 = exp (-t/T);
			y2 = 1 + floor(100 Hz * t/T) - (100 Hz * t/T);
			y3 = 1;

			output (y0, y1, y2, y3);
		}

	/* Noise */
	}else if (!strcmp (type, "--noise")){

		comment = "cos waves, with various amounts of noise";
		eqn0   =  "y0 = cos(w t) + Noise,         f = 1 kHz";
		eqn1   =  "y1 = 0.1 cos(w t) + Noise,     f = 1 kHz";
		eqn2   =  "y2 = cos (w t) + 0.1 * Noise,  f = 1 kHz";
		eqn3   =  "y3 = 0.01 * cos (w t) + Noise, f = 1 kHz";

		add_titles (comment, eqn0, eqn1, eqn2, eqn3);

		for (i = 0 ; i < SAMPLES; i++){
			tpt = (double)(2 * PI * i * PERIOD_US) / 1e6;
			wt  = 1 kHz * tpt;

			y0 = cos (wt) + noise();
			y1 = 0.1 * cos (wt) + noise();
			y2 = cos (wt) + 0.1 * noise();
			y3 = 0.01 * cos (wt) + noise();

			output (y0, y1, y2, y3);
		}

	/* DC, with Noise and Hum */
	}else if (!strcmp (type, "--dchum")){

		comment = "dc level, with noise and hum";
		eqn0   =  "y0 = 1                                   ";
		eqn1   =  "y1 = 2 + 0.1 * Noise                     ";
		eqn2   =  "y2 = 3 + 0.001 * Noise                   ";
		eqn3   =  "y3 = 10 + cos (w t) + Noise,    f = 1 kHz";

		add_titles (comment, eqn0, eqn1, eqn2, eqn3);

		for (i = 0 ; i < SAMPLES; i++){
			tpt = (double)(2 * PI * i * PERIOD_US) / 1e6;
			wt  = 1 kHz * tpt;

			y0 = 1;
			y1 = 2  + 0.1 * noise();
			y2 = 3  + 0.001 * noise();
			y3 = 10 + cos (wt) + noise();

			output (y0, y1, y2, y3);
		}


	}else{

		fprintf (stderr, "Sorry, format %s not recognised. Use -h for list.\n", type);
		exit(1);
	}


	/* End */
	fclose ( outfile );
	return (0);
}
